@@ -113,4 +113,4 @@ print_argc_vars() {
     ( set -o posix ; set ) | grep argc_
 }
 
-eval $(target/debug/argc -e $0 "$@")
\ No newline at end of file
+eval $(target/debug/argc $0 "$@")
\ No newline at end of file
