@@ -2,5 +2,5 @@
 mod macros;
 mod escape_test;
 mod fail_test;
-mod mainfn_test;
+mod main_fn_test;
 mod spec_test;
